--[[ borlox 2009-05-01T14:46:39Z 73d75bb ]]--

local _G = _G

local NUM_CHAR_FRAME_TABS = 5
local NUM_SIDE_FRAME_TABS = 5
local PLAYER_FACTION_ID = UnitFactionGroup("player") == "Horde" and 1 or 2

TabbyDB = {
	tabButtonList = {
		1, 2, 3, 4, 5
	}	
}

--------

local buttonInfo = {
	[1] = {
		tooltip     = MicroButtonTooltipText(CHARACTER_INFO, "TOGGLECHARACTER0"),
		frameToShow = "PaperDollFrame",
		texture     = function(btn)
			if btn.SetNormalTexture then
				btn:SetNormalTexture("")
			end
			SetPortraitTexture(btn.tex, "player")
		end,
	},
	[2] = {
		tooltip     = MicroButtonTooltipText(PET, "TOGGLECHARACTER3"),
		frameToShow = "PetPaperDollFrame",
		texture     = function(btn) 
			local t
			if HasPetUI() then
				t = GetPetIcon()
			elseif GetNumCompanions("MOUNT") > 0 then
				t = select(4, GetCompanionInfo("MOUNT", 1))
			else
				t = "Interface\\Icons\\Ability_Seal"
			end
			if btn.SetNormalTexture then
				btn:SetNormalTexture(t)
			else
				btn.tex:SetTexture(t)
			end
		end,
	},	
	[3] = {
		tooltip     = MicroButtonTooltipText(REPUTATION, "TOGGLECHARACTER2"),
		frameToShow = "ReputationFrame",
		texture     = "Interface\\Icons\\INV_BannerPVP_0" .. PLAYER_FACTION_ID,
	},
	[4] = {
		tooltip     = MicroButtonTooltipText(SKILLS, "TOGGLECHARACTER1"),
		frameToShow = "SkillFrame",
		texture     = "Interface\\Icons\\INV_Scroll_07",
	},
	[5] = {
		tooltip     = MicroButtonTooltipText(TOKENS, "TOGGLECHARACTER4"),
		frameToShow = "TokenFrame",
		texture     = "Interface\\Icons\\INV_Jewelry_Talisman_01",
	},
}

local sideBarButtons = {}
local dropFrames = {}

local dragFrame
local flashFrame

local tabButtonList

local isDragging, draggedId

--------

local UpdateButtons

local function HideCharFrameTabs()
	for i=1, NUM_CHAR_FRAME_TABS do
		_G["CharacterFrameTab"..i]:SetScript("OnShow", function(self) self:Hide() end)
	end
end

local function PlaceIdInPosition(id, pos)
	for i=#tabButtonList, pos, -1 do
		tabButtonList[i+1] = tabButtonList[i]
	end
	tabButtonList[pos] = id
	UpdateButtons()
end

local function RemovePosition(pos)
	for i=pos+1, #tabButtonList do
		tabButtonList[i-1] = tabButtonList[i]
	end
	tabButtonList[#tabButtonList] = nil
	UpdateButtons()
end

local function SideTab_OnClick(self, button)
	local id = self:GetID()
	ToggleCharacter(buttonInfo[id].frameToShow)
	PlaySound("igCharacterInfoTab")
	
	--flashFrame:SetPoint("CENTER", self, "CENTER")
	--flashFrame:Show()
end

local function SideTab_OnEnter(self)
	local id = self:GetID()
	GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
	GameTooltip:SetText(buttonInfo[id].tooltip, 1, 1, 1)
end

local function SideTab_OnLeave(self)
	GameTooltip:Hide()
end

local function SideTab_OnDragStart(self)
	local id = self:GetID()
	local info = buttonInfo[id]
	
	local texType = type(info.texture)
	if texType == "string" then
		dragFrame.tex:SetTexture(info.texture)
	elseif texType == "function" then
		info.texture(dragFrame)
	else
		assert(false)
	end
	
	isDragging = true
	draggedId  = id
	
	RemovePosition(self.pos)
		
	dragFrame:Show()
end

local function Drop_OnEnter(self)
	if isDragging then
		self.highlight:Show()
	end
end

local function Drop_OnLeave(self)
	self.highlight:Hide()
end

local function Drop_OnReceiveDrag(self)
	if isDragging then
		isDragging = false
		dragFrame:Hide()
		
		PlaceIdInPosition(draggedId, self.pos)
	end	
end

local function DragFrame_OnUpdate(self)
	local x, y = GetCursorPosition(UIParent)
	local scale = UIParent:GetScale()
	self:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", x/scale, y/scale)
	self:Show()
end

local function CreateDragFrame()
	dragFrame = CreateFrame("Frame", nil, UIParent)
	dragFrame:SetHeight(32)
	dragFrame:SetWidth(32)

	dragFrame:SetScript("OnUpdate", DragFrame_OnUpdate)

	local tex = dragFrame:CreateTexture("BACKGROUND")
	tex:SetAllPoints(dragFrame)
	dragFrame.tex = tex

	tex:Show()
	dragFrame:Hide()
end

local function CreateFlashFrame()
--	flashFrame = CreateFrame("Frame", "CharacterFrameSideTabFlash", CharacterFrame)
--	flashFrame:SetWidth(64)
--	flashFrame:SetHeight(64)
--	flashFrame:SetAlpha(.6)
--
--	flashFrame.flash = flashFrame:CreateTexture("OVERLAY")
--	flashFrame.flash:SetTexture("Interface\\Buttons\\CheckButtonGlow")
--	flashFrame.flash:SetBlendMode("ADD")
--	flashFrame.flash:SetAllPoints(flashFrame)
--	flashFrame.flash:Show()
--
--	flashFrame:Hide()
end

local function CreateButtons()
	for i=1, NUM_SIDE_FRAME_TABS do
		local btn = CreateFrame("Button", "CharacterFrameSideTab"..i, CharacterFrame, "SpellBookSkillLineTabTemplate")
		sideBarButtons[i] = btn
		
		btn.pos = i

		btn:SetScript("OnClick", SideTab_OnClick)
		btn:SetScript("OnEnter", SideTab_OnEnter)
		btn:SetScript("OnLeave", SideTab_OnLeave)
		btn:SetScript("OnDragStart", SideTab_OnDragStart)
		btn:RegisterForDrag("LeftButton")

		local tex = btn:CreateTexture("NORMAL")
		tex:SetAllPoints(btn)
		btn.tex = tex

		if i == 1 then
			btn:SetPoint("TOPLEFT", CharacterFrame, "TOPRIGHT", -32, -65)
		else
			btn:SetPoint("TOPLEFT", sideBarButtons[i-1], "BOTTOMLEFT", 0, -17)
		end

		tex:Show()
		btn:Show()
		
		local drop = CreateFrame("Frame", "CharacterFrameSideTabDrop"..i, CharacterFrame)
		drop:SetHeight(8)
		drop:SetWidth(btn:GetWidth())
		drop:SetPoint("BOTTOM", btn, "TOP", 0, 5)
		drop:EnableMouse(true)
		
		drop.pos = i
				
		local highlight = drop:CreateTexture("BACKGROUND")
		highlight:SetTexture("Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight") --1, 1, 0, .1)
		highlight:SetAlpha(.7)
		highlight:SetAllPoints(drop)
		drop.highlight = highlight
		highlight:Hide()
		
		drop:SetScript("OnEnter", Drop_OnEnter)
		drop:SetScript("OnLeave", Drop_OnLeave)
		drop:SetScript("OnMouseUp", Drop_OnReceiveDrag)
		drop:SetScript("OnReceiveDrag", Drop_OnReceiveDrag)
		drop:Show()
	end
end

function UpdateButtons()
	for i=1, NUM_SIDE_FRAME_TABS do
		local btn = sideBarButtons[i]
		local id  = tabButtonList[i]
		
		if id then
			local info = buttonInfo[id]
			
			btn:SetID(id)
			local texType = type(info.texture)
			if texType == "string" then
				btn:SetNormalTexture(info.texture)
				btn.tex:Hide()
			elseif texType == "function" then
				info.texture(btn)
				btn.tex:Show()
			else
				assert(false)
			end
			
			btn:Show()
		else
			btn:Hide()
		end	
	end
end

--------

local Tabby = CreateFrame("Frame", "Tabby")

function Tabby:OnEvent(event, ...)
	if self[event] then
		self[event](self, ...)
	end
end

function Tabby:ADDON_LOADED(addon)
	if addon == "Tabby" then
		tabButtonList = TabbyDB.tabButtonList

		HideCharFrameTabs()
		CreateDragFrame()
		CreateFlashFrame()
		CreateButtons()
		UpdateButtons()	

		self:RegisterEvent("COMPANION_UPDATE")
	end
end

function Tabby:COMPANION_UPDATE()
	UpdateButtons()
end

Tabby:SetScript("OnEvent", Tabby.OnEvent)
Tabby:RegisterEvent("ADDON_LOADED")
